package com.loafle.overflow.module.noauthprobe.service;

import com.loafle.overflow.module.apikey.model.ApiKey;
import com.loafle.overflow.module.apikey.service.ApiKeyService;
import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaNoAuthProbeStatus;
import com.loafle.overflow.module.meta.model.MetaProbeStatus;
import com.loafle.overflow.module.noauthprobe.dao.NoAuthProbeDAO;
import com.loafle.overflow.module.noauthprobe.model.NoAuthProbe;
import com.loafle.overflow.module.probe.model.Probe;
import com.loafle.overflow.module.probe.service.ProbeService;
import com.loafle.overflow.commons.service.MessagePublisher;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("NoAuthProbeService")
public class NoAuthProbeService {

    @Autowired
    private NoAuthProbeDAO noAuthProbeDAO;

    @Autowired
    private ApiKeyService apiKeyService;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private MessagePublisher messagePublisher;

    public NoAuthProbe regist(NoAuthProbe noAuthProbe) {

        noAuthProbe.setTempProbeKey(UUID.randomUUID().toString());
        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short)3));

        ApiKey apiKey = apiKeyService.readByApiKey(noAuthProbe.getApiKey());
        noAuthProbe.setDomain(apiKey.getDomain());

        messagePublisher.publishToDomain("/web", apiKey.getDomain(), "NoAuthProbeService.regist", noAuthProbe);

        return this.noAuthProbeDAO.save(noAuthProbe);
    }


    public List<NoAuthProbe> readAllByDomain(Domain domain) {

        return this.noAuthProbeDAO.findAllByDomain(domain);
    }

    public NoAuthProbe read(long id) {
        return this.noAuthProbeDAO.findOne(id);
    }

    public List<NoAuthProbe> acceptNoAuthProbe(NoAuthProbe noAuthProbe) throws IOException {

        // Todo domain injection & member injection


        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        String encryptKey = passwordEncoder.encode(UUID.randomUUID().toString());

        Probe probe = new Probe();
        probe.setEncryptionKey(encryptKey);
        probe.setProbeKey(noAuthProbe.getApiKey());
        probe.setDomain(new Domain(1));
        probe.setAuthorizeMember(new Member(1));
        probe.setStatus(new MetaProbeStatus((short)1));
        probe.setTargetCount(0);
        probe.setSensorCount(0);

        Map<String, Object> objMap = this.objectMapper.readValue(noAuthProbe.getDescription(), new TypeReference<HashMap<String,Object>>() {});
        Map<String, String> hostMap = (Map<String, String>)objMap.get("host");
        Map<String, String> netMap = (Map<String, String>)objMap.get("network");

//            String dispName = noAuth.getHostName().isEmpty() ?
//                    StringConvertor.intToIp(noAuth.getIpAddress()) : noAuth.getHostName();
        String dispName = hostMap.get("name");
        dispName += " probe";
        probe.setDisplayName(dispName);

        String addrStr = netMap.get("address");

        String[] addrArr = addrStr.split("\\|");

        probe.setCidr(addrArr[0]);

        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short) 1));

        this.probeService.regist(probe);
        this.noAuthProbeDAO.save(noAuthProbe);

        messagePublisher.publishToProbe("/auth", noAuthProbe.getTempProbeKey(), "NoAuthProbeService.acceptNoAuthProbe", probe.getProbeKey());

        return this.readAllByDomain(noAuthProbe.getDomain());
    }

    public List<NoAuthProbe> denyNoauthProbe(NoAuthProbe noAuthProbe) {
        this.noAuthProbeDAO.save(noAuthProbe);

        messagePublisher.publishToProbe("/auth", noAuthProbe.getTempProbeKey(), "NoAuthProbeService.denyNoauthProbe");

        return this.readAllByDomain(noAuthProbe.getDomain());
    }

    public NoAuthProbe readByTempKey(String tempKey) {
        return this.noAuthProbeDAO.findByTempProbeKey(tempKey);
    }
}
